#pragma once

// CS559-2007 - Project 1 - Picture Processor
//
// Example Solution, by Mike Gleicher
// begin 9/10/2007

// file: PictureProcessor.H
// contains the header for basic handling of the script files. it does
// not have any of the image handling inside of it

// i will use the STL and old fashioned "C" file I/O
// i will also use C strings, not C++ strings
#include <vector>

// these basic utilities do not know about images, but they do need to
// manipulate pointers to them. we will use the LibTargaWrapper
class TargaImage;

// utility for breaking a line into a set of words
// note: this does not copy the input string, it does break it up by
// modifying it (sticking end of string markers into it)
//
// we return a vector of pointers into the string, each one pointing
// to the beginning of a word
// the number of words is the final size of the vector
//
void breakString(char* str, std::vector<const char*>& words);

// the "context" for the interpetter - basically stores the values of
// the variables, and keeps track of where in the script file you are
// (for error handling)
class Context {
 public:
  Context();					/* create a new one */
  ~Context();					/* delete - remember to get rid of any
								   images stored in variables! */

  int lineNo;					/* which line are we on */

  // we guard the access to the variables, because we want to make
  // sure that when you set one, that you delete the old contents
	void setVar(const int, TargaImage*);
	TargaImage* getVar(const int) const;

	///////// helpers for parsing
	// helper for parsing - given a string that we expect to be a variable name
	// return the index (0-25) of the variable - or return -1 if the string
	// is not a valid variable name. this is a method so it can print a proper
	// error message.
	int variableIndex(const char* v) const;

	// given a string, get the image in the variable (named in the string)
	// prints error messages if things are bad
	TargaImage* getImage(const char* v) const;

 private:
  // store the variables - these either contain a null pointer or a
  // pointer to a valid image (that is "owned" by this context)
  std::vector<TargaImage*> variables;
};

// we want to encapsulate functions that perform the commands in a
// class
// each command will be a subclass of this class
// using a trick, we'll declare a global instance and have it install
// itself on a linked list of all the installed commands - that way we
// can simply add commands by adding new .cpp files to the project!
// no one has to know what commands we have at compile time!
//
// in order to support parsing, each command needs to know:
//    the command name (the "verb" associated with it)
//    the number of arguments that it should take (INCLUDES the
//    command)
//
// the actual work is done in the "doit" method - this is where a
// command is processed, and variables (images) get updated accordingly
class Command {
 public:
  Command(const char* verb, int nargs);
  // remember for parsing
  const char* verb;
  int nargs;
  // this is the thing that does the work - it returns non-zero if
  // there is an error (zero means no error)
  virtual int doit(const std::vector<const char*>& cmd,
				   Context& context) = 0;

  // we'll keep a linked list of the commands, since that way it
  // doesn't matter what order they are instantiated in (an issue we'd
  // have if we were using an STL vector)
  Command* next;
};


// the head of the list (global) of the command
extern Command* commands;
