// CS559-2007 - Project 1 - Picture Processor
//
// Example Solution, by Mike Gleicher
// begin 9/10/2007

// file: ReadWrite.cpp
// implements the reading and writing commands 
// (and the command for making a blank image)

#include "PictureProcessor.H"
#include "TargaImage.H"
#include "libtarga.h"

//////////////////
// this command reads in an image
// syntax: read V filename
class ReadCmd : public Command
{
public:
	ReadCmd() : Command("read",3) {};
	virtual int doit(const std::vector<const char*>& cmd,
					 Context& context) {
			int var = context.variableIndex(cmd[1]);
			// variable index prints an error message for us - so we can just return
			if (var < 0) return 1;

			// ok, let's try to read in the image!
			TargaImage* img = TargaImage::readImage(cmd[2]);

			if (!img) {
				printf("Error (line %d): can't read image! (%s)\n",context.lineNo,tga_error_string(tga_get_last_error()));
				return 2;
			}
			context.setVar(var,img);
			return 0;
	};
};
// instantiate one of these
ReadCmd readCmd;

////////////////////
// this command writes an image back out to a TGA file
// syntax write V filename
class WriteCmd : public Command
{
public:
	WriteCmd() : Command("write",3) {};
	virtual int doit(const std::vector<const char*>& cmd,
					 Context& context) {
			TargaImage* img = context.getImage(cmd[1]);
			if (!img) {
				printf("Error (line %d): no image to write!\n",context.lineNo,tga_error_string(tga_get_last_error()));
				return 2;
			} else {
				img->write(cmd[2]);
			}
			return 0;
	};
};
// instantiate one of these
WriteCmd writeCmd;

//
// the blank command creates a black image of size xy
// syntax blank V x y
class BlankCmd : public Command
{
public:
	BlankCmd() : Command("blank",4) {};
	virtual int doit(const std::vector<const char*>& cmd,
					 Context& context) {
		// get all of the parameters first, and make sure they are valid
		// before allocating the new image
		 int var = context.variableIndex(cmd[1]);
		 int sx = atoi(cmd[2]);
		 int sy = atoi(cmd[3]);
		 // bad variable, we've already gotten an error message
		 if (var < 0) return 1;
		 // check for a valid size
		 if (sx<1 || sx>65535 || sy<1 || sy>65535) {
			 printf("Error (line %d): invalid size for blank image\n",context.lineNo);
			 return 2;
		 }
		 TargaImage* img = TargaImage::blankImage(sx,sy);
		 if (!img) {
			 printf("Error (line %d): couldn't create blank image\n",context.lineNo);
			 return 3;
		 }
		 img->fillAlpha(255);
		 context.setVar(var,img);
		 return 0;
	};
};
// instance one of these
BlankCmd blankCmd;