// CS559-2007 - Project 1 - Picture Processor
//
// Example Solution, by Mike Gleicher
// begin 9/10/2007

// file: main.cpp
// 
// main execution loop

#include "PictureProcessor.H"
#include "stdio.h"

int main(int argc, char** argv)
{
	printf("Picture Processor\n");

	// look at the arguments - if there is a file, then we'll read from that,
	// otherwise, we'll read from stdin
	FILE* scriptInput = stdin;
	if (argc == 1) {
		printf("Reading from STDIN\n");
		printf("Installed commands: ");
		for(Command* c = commands; c != 0; c=c->next)
			printf("%s ",c->verb);
		printf("\n");
	} else if (argc == 2) {
		// note: using the old-fashioned "fopen" gives a warning from VS2005
		// i like my code to compile with no warning, so I do what they want
		printf("Opening script file %s\n",argv[1]);
		errno_t err;
		if (err=fopen_s(&scriptInput,argv[1],"r")) {
			printf("Can't open file `%s' - error %d\n",argv[1],err);
		}
	} else {
		printf("PictureProcessor requires either 0 or 1 command line argument!");
		exit(1);
	}
	if (!scriptInput) {
		printf("Failed to open script file!\n");
		exit(1);
	}

	// make a context to store all the images and whatnot
	Context context;

	// read the input, a line at a time until we get to the end of the file
	char buf[256];	// lines longer than this will not be handled
	while(fgets(buf,256,scriptInput)) {
		// keep count of the line we were on (for error reporting)
		context.lineNo++;

		// turn the command into a set of words
		std::vector<const char*> words;
		breakString(buf,words);

		// use this for debugging - make sure things got broken into words
		// correctly
#ifdef TESTBREAKWORDS
		printf("%d words:\n",words.size());
		for(size_t i=0; i<words.size(); i++)
			printf("%s ",words[i]);
		printf("\n");
#endif // TESTBREAKWORDS
		// if there is a command on the line, then figure out which it is
		if (words.size()) {
			// loop over the commands
			Command* c;
			for(c = commands; c ; c = c->next)
				if (!_stricmp(c->verb,words[0])) 
					break;
			// did we find a match?
			if (c) {
				// check to make sure there is the right number of parameters
				if (c->nargs == words.size()) {
					c->doit(words,context);
				} else
					printf("Error (line %d): wrong number of arguments to command `%s' (expecting %d got %d)\n",context.lineNo,c->verb,c->nargs,words.size());
			} else {
				printf("Error (line %d): unknown verb `%s'\n",context.lineNo,words[0]);
			}
		}
	}

	// we're done! close the file
	if (scriptInput != stdin) fclose(scriptInput);
	return 0;
}